"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlgorithmBase = void 0;
class AlgorithmBase {
    constructor(_bestChanged = undefined) {
        this._bestChanged = _bestChanged;
        this._chromosomes = [];
        this._fitness = 0;
    }
    get generationNumber() {
        // eslint-disable-next-line no-magic-numbers
        return this.islands.reduce((acc, island) => acc + island.generationNumber, 0);
    }
    get offspringNumber() {
        // eslint-disable-next-line no-magic-numbers
        return this.islands.reduce((acc, island) => acc + island.offspringNumber, 0);
    }
    get initialized() {
        return this.islands.every(island => island.initialized);
    }
    get chromosomes() {
        return this._chromosomes;
    }
    get migration() {
        return undefined;
    }
    get best() {
        return this._chromosomes[0];
    }
    get progress() {
        return this.termination.progress;
    }
    get hasReached() {
        return this.termination.hasReached(this);
    }
    async updateChromosomes() {
        var _a;
        // eslint-disable-next-line no-magic-numbers
        this._chromosomes = this.islands.flatMap(island => island.population.chromosomes).sort((c1, c2) => { var _a, _b; return ((_a = c2.fitness) !== null && _a !== void 0 ? _a : -1.0) - ((_b = c1.fitness) !== null && _b !== void 0 ? _b : -1.0); });
        // eslint-disable-next-line no-magic-numbers
        const bestFitness = (_a = this.best.fitness) !== null && _a !== void 0 ? _a : -1;
        // eslint-disable-next-line no-magic-numbers
        if (bestFitness >= 0 && bestFitness !== this._fitness) {
            this._fitness = bestFitness;
            if (this._bestChanged) {
                await this._bestChanged();
            }
        }
    }
    async reset() {
        var _a;
        await Promise.all(this.islands.map(island => island.reset()));
        (_a = this.migration) === null || _a === void 0 ? void 0 : _a.init();
        this.termination.init();
        await this.updateChromosomes();
        this.performReset();
    }
    performReset() {
        // override if required
    }
    async step() {
        var _a;
        if (this.hasReached) {
            return;
        }
        await Promise.all(this.islands.map(island => island.step()));
        await ((_a = this.migration) === null || _a === void 0 ? void 0 : _a.migrate(this));
        this.updateChromosomes();
        this.performStep();
    }
    performStep() {
        // override if required
    }
}
exports.AlgorithmBase = AlgorithmBase;
//# sourceMappingURL=AlgorithmBase.js.map